def importCode(code,name,add_to_sys_modules=0):
    """
    Import dynamically generated code as a module. code is the
    object containing the code (a string, a file handle or an
    actual compiled code object, same types as accepted by an
    exec statement). The name is the name to give to the module,
    and the final argument says wheter to add it to sys.modules
    or not. If it is added, a subsequent import statement using
    name will return this module. If it is not added to sys.modules
    import will try to load it in the normal fashion.

    import foo

    is equivalent to

    foofile = open("/path/to/foo.py")
    foo = importCode(foofile,"foo",1)

    Returns a newly generated module.
    """
    import sys,imp

    mod = imp.new_module(name)

    exec code in mod.__dict__
    if add_to_sys_modules:
        sys.modules[name] = mod

    return mod



from django.db import models
from django.utils.translation import ugettext_lazy

from django_reports.utils import importCode


class Report(models.Model):
    name = models.CharField(max_length=10, unique=True, verbose_name=ugettext_lazy("Name"))
    title = models.CharField(max_length=100, verbose_name=ugettext_lazy("Name"))
    description = models.CharField(max_length=500, verbose_name=ugettext_lazy("Name"))
    source_code = models.CharField(max_length=4000, verbose_name=ugettext_lazy("Source Code"))
    style = models.CharField(max_length=10, verbose_name=ugettext_lazy("Name"))

    def __init__(self, *args, **kwargs):
        self.query_module = None
        super(Report, self).__init__(*args, **kwargs)

    def compile(self):
        self.query_module = importCode(self.source_code, "query_module")
        print(self.query_module)

    def eval(self, **kwargs):
        if self.query_module:
            return self.query_module.query._eval(**kwargs)
        else:
            raise RuntimeError("Report must be compiled before evaluation")

    def get_form(self, **kwargs):
        if self.query_module:
            return self.query_module.query._get_form(**kwargs)
        else:
            raise RuntimeError("Report must be compiled before evaluation")



import json

from abc import abstractmethod, ABCMeta
from django.core import serializers
from django.db import models
from django.http.response import HttpResponse
from django.views.generic.base import View

from django_reports.models import Report


class ReportQuery:
    """
    Abstract class that must be overridden for every report. Each report must implement ``eval`` and ``get_form``. 
    ``eval`` must provide the input for highcharts as python dictionary. ``get_form`` must provide the name and 
    configurations for each parameter including the choices
    """

    __metaclass__ = ABCMeta

    @abstractmethod
    def eval(self, **kwargs):
        pass

    def _eval(self, **kwargs):
        objects = self.eval(**kwargs)
        return objects

    @abstractmethod
    def get_form(self, **kwargs):
        pass

    def _get_form(self, **kwargs):
        objects = self.get_form(**kwargs)
        return objects


class ReportView(View):

    def get(self,request):
        if "report" in request.GET and "get" in request.GET :
            report = Report.objects.get(name=request.GET["report"])
            report.compile()
            params = json.loads(request.GET["parameters"]) if "parameters" in request.GET else dict()
            if request.GET["get"] == "form":
                return render_to_json_response(report.get_form(**params))
            elif request.GET["get"] == "values":
                return render_to_json_response(report.eval(**params))
        else:
            return render_to_json_response(
                [
                    {
                        "name":r.name,
                        "title":r.title,
                        "style":r.style,
                        "description":r.description
                    }
                    for r in
                    Report.objects.all()
                ]
            )


def render_to_json_response(obj, **response_kwargs):
    global data
    if isinstance(obj, models.QuerySet):
        data = serializers.serialize('json', obj)
    else:
        data = json.dumps(obj)
    response_kwargs['content_type'] = 'application/json'
    return HttpResponse(data, **response_kwargs)